package dev.jansel.aglaea

import com.github.twitch4j.TwitchClient
import dev.jansel.aglaea.extensions.EventHooks
import dev.jansel.aglaea.extensions.ReplayExtension
import dev.jansel.aglaea.extensions.SessionExtension
import dev.jansel.aglaea.utils.database
import dev.jansel.aglaea.utils.twitch
import dev.kord.common.entity.Snowflake
import dev.kordex.core.ExtensibleBot
import dev.kordex.core.utils.env
import io.github.oshai.kotlinlogging.KotlinLogging
import java.io.File

var twitchClient: TwitchClient? = null
val logger = KotlinLogging.logger { }
var botRef: ExtensibleBot? = null

val TEST_SERVER_ID = Snowflake(
	env("TEST_SERVER").toLong()  // Get the test server ID from the env vars or a .env file
)

private val TOKEN = env("TOKEN")   // Get the bot' token from the env vars or a .env file

suspend fun main() {
	botRef = ExtensibleBot(TOKEN) {
		extensions {
			add(::ReplayExtension)
			add(::EventHooks)
			add(::SessionExtension)
		}
		database(true)
		twitch(true)
		if (devMode) {
			// In development mode, load any plugins from `src/main/dist/plugin` if it exists.
			plugins {
				if (File("src/main/dist/plugins").isDirectory) {
					pluginPath("src/main/dist/plugins")
				}
			}
		}
	}

	botRef!!.start()
}
