package dev.jansel.aglaea.extensions

import dev.jansel.aglaea.botRef
import dev.jansel.aglaea.database.collections.ConfigCollection
import dev.jansel.aglaea.database.collections.ReplayCollection
import dev.jansel.aglaea.database.entities.ReplayData
import dev.jansel.aglaea.i18n.Translations
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.behavior.getChannelOf
import dev.kord.core.entity.channel.GuildMessageChannel
import dev.kordex.core.checks.inChannel
import dev.kordex.core.commands.Arguments
import dev.kordex.core.commands.converters.impl.attachment
import dev.kordex.core.extensions.Extension
import dev.kordex.core.extensions.ephemeralSlashCommand
import dev.kordex.core.utils.download
import io.ktor.client.request.forms.*
import io.ktor.utils.io.*
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import kotlin.uuid.ExperimentalUuidApi
import kotlin.uuid.Uuid

class ReplayExtension : Extension() {
	override val name = "test"

	@OptIn(ExperimentalUnsignedTypes::class, ExperimentalUuidApi::class)
	override suspend fun setup() {
		ephemeralSlashCommand(::ReplayArgs) {
			name = Translations.Commands.Replay.name
			description = Translations.Commands.Replay.description
			check {
				inChannel(Snowflake(1130954956892029060))
			}
			action {
				val file = arguments.file
				val downloadedFile = file.download()
				val replayToggle = ConfigCollection().get()?.replaySession ?: false

				if (replayToggle) {
					val channel = botRef!!.kordRef.getGuildOrNull(Snowflake(1130932838200512584))!!
						.getChannelOf<GuildMessageChannel>(Snowflake(1130954956892029060))
					runBlocking {
						launch {
							channel.createMessage {
								content = "Replay submitted by: ${event.interaction.user.globalName}"
								this.addFile("replay.osr", ChannelProvider(null) { ByteReadChannel(downloadedFile) })
							}
						}
					}
					respond {
						content = "Replay submitted successfully!"
					}
					return@action
				}
				if (file.filename.endsWith(".osr")) {
					val id = Uuid.random().toString()

					ReplayCollection().set(ReplayData(id, file.download()))
					respond {
						content = "Your Auth Code is: $id"
					}
				} else {
					respond {
						content = ""
					}
				}
			}
		}
	}

	inner class ReplayArgs : Arguments() {
		val file by attachment {
			name = Translations.Arguments.File.name
			description = Translations.Arguments.File.description
		}
	}
}
