package dev.jansel.aglaea.utils

import dev.jansel.aglaea.database.Database
import dev.jansel.aglaea.database.collections.ConfigCollection
import dev.jansel.aglaea.database.collections.MetaCollection
import dev.jansel.aglaea.database.collections.ReplayCollection
import dev.jansel.aglaea.twitchClient
import dev.kordex.core.builders.ExtensibleBotBuilder
import dev.kordex.core.utils.env
import dev.kordex.core.utils.loadModule
import kotlinx.coroutines.runBlocking
import org.koin.dsl.bind

val twitchcid = env("TWITCH_CLIENT_ID")
val twitchcs = env("TWITCH_CLIENT_SECRET")
val twitchToken = env("TWITCH_OAUTH_TOKEN")
val mongoUri = env("MONGO_URI")

suspend inline fun ExtensibleBotBuilder.database(migrate: Boolean) {
	val db = Database()

	hooks {
		beforeKoinSetup {
			loadModule {
				single { db } bind Database::class
			}

			loadModule {
				single { MetaCollection() } bind MetaCollection::class
				single { ReplayCollection() } bind ReplayCollection::class
				single { ConfigCollection() } bind ConfigCollection::class
			}

			if (migrate) {
				runBlocking { db.migrate() }
			}
		}
	}
}

suspend inline fun ExtensibleBotBuilder.twitch(active: Boolean) {
	hooks {
		beforeKoinSetup {
			loadModule {
				single { Twitch() } bind Twitch::class
			}

			if (active) {
				Twitch().init()
			}
		}
	}
}

fun getTwitchNameById(id: String): String? {
	val resultList = twitchClient!!.helix?.getUsers(null, listOf(id), null)?.execute()
	resultList?.users?.forEach { user ->
		if (user.id == id) {
			return user.displayName
		}
	}
	return null
}
